<?php

/*
 *  Copyright (c) 2012  Rasmus Fuhse <fuhse@data-quest.de>
 * 
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 */

require_once dirname(__file__)."/classes/st_score.class.php";

class SurvivalTrainingPlugin extends StudIPPlugin implements StandardPlugin {
    
    public function __construct() {
        parent::__construct();
        if (Navigation::hasItem("/course") && (!$_SESSION['SessionSeminar'] || $this->isActivated())) {
            $navigation = new Navigation($this->getDisplayName(), PluginEngine::getURL($this, array(), 'survive'));
            $navigation->setImage($this->getPluginURL()."/assets/fighter_white.png");
            Navigation::addItem("/course/survivaltraining", $navigation);

            $navigation = new AutoNavigation($this->getDisplayName(), PluginEngine::getURL($this, array(), 'survive'));
            Navigation::addItem("/course/survivaltraining/play", $navigation);

            $navigation = new AutoNavigation(_("Ranking"), PluginEngine::getURL($this, array(), 'highscore'));
            Navigation::addItem("/course/survivaltraining/highscore", $navigation);
        }
    }
    
    public function getDisplayName() {
        return _("Survival-Training");
    }
    
    public function survive_action() {
        Navigation::getItem("/course/survivaltraining")->setImage($this->getPluginURL()."/assets/fighter_black.png");
        PageLayout::addHeadElement('link', array('href'=>$this->getPluginURL()."/cron_game/assets/css/style.css",'rel'=>'stylesheet','media'=>'screen, print'));
        
        PageLayout::addHeadElement('script', array(), 'game_path = "'.$this->getPluginURL().'/cron_game/";');
        
        $scripts = array("crafty","game","effects","enemys","levels","player","sounds","sprites","weapons");
        foreach ($scripts as $script) {
            PageLayout::addHeadElement('script', array('src'=>$this->getPluginURL()."/cron_game/assets/js/$script.js"),"");
        }
        
        $template = $this->getTemplate("survive.php");
        $template->set_attribute('save_score_url', PluginEngine::getURL($this,array(),'save_score'));
        echo $template->render();
    }
    
    public function save_score_action() {
        if (!$GLOBALS['perm']->have_perm("autor") || !$_SESSION['SessionSeminar'] || !$GLOBALS['perm']->have_studip_perm("autor", $_SESSION['SessionSeminar'])) {
            throw new AccessDeniedException("Kein Zugriff");
        }
        if (Request::get("score")) {
            st_score::insertScore(Request::get("score"));
        }
    }
    
    public function highscore_action() {
        Navigation::getItem("/course/survivaltraining")->setImage($this->getPluginURL()."/assets/fighter_black.png");
        
        $template = $this->getTemplate("highscore.php");
        $template->set_attribute('score_list', st_score::findBySQL("st_score", "seminar_id = ".DBManager::get()->quote($_SESSION['SessionSeminar'])." ORDER BY score DESC"));
        $template->set_attribute('plugin', $this);
        $template->set_attribute('global_score_list', json_decode(file_get_contents(st_score::getServer()."plugins.php/survivaltrainingplugin/central_score")));
        echo $template->render();
    }
    
    /**
     * wird nur aufgerufen, wenn wir nicht der zentrale Server sind. 
     */
    public function receive_score_action() {
        $output = array();
        if (Request::get('secret') === st_score::getSecret()) {
            $score = new st_score(Request::get("score_id"));
            $score['user_id'] = Request::get("user_id");
            $score['user_name'] = Request::get("user_name");
            $score['seminar_id'] = Request::get("seminar_id");
            $score['seminar_name'] = Request::get("seminar_name");
            $score['server_adress'] = Request::get("server_adress");
            $score['server_name'] = Request::get("server_name");
            $score['score'] = Request::int("score");
            $score->store();
            $output['success'] = studip_utf8encode(_("Daten eingerichtet."));
        } else {
            $output['error'] = studip_utf8encode(_("Falsches Passwort."));
        }
        echo json_encode($output);
    }
    
    public function central_score_action() {
        $list = st_score::findBySQL("st_score", "1=1 ORDER BY score DESC, mkdate ASC LIMIT 30");
        foreach ($list as $key => $entry) {
            $list[$key] = array(
                'st_score_id' => $entry['st_score_id'],
                'user_id' => $entry['user_id'],
                'user_name' => studip_utf8encode($entry['user_name']),
                'server_adress' => studip_utf8encode($entry['server_adress']),
                'server_name' => studip_utf8encode($entry['server_name']),
                'seminar_id' => $entry['seminar_id'],
                'seminar_name' => studip_utf8encode($entry['seminar_name']),
                'score' => (int) $entry['score'],
                'mkdate' => (int) $entry['mkdate']
            );
        }
        echo json_encode($list);
    }
    
    public function getIconNavigation($course_id, $last_visit) {
        $db = DBManager::get();
        
        $navigation = new Navigation($this->getDisplayName(), PluginEngine::getURL($this, array(), 'survive'));
        $my_score = $db->query(
            "SELECT score " .
            "FROM st_score " .
            "WHERE seminar_id = ".$db->quote($course_id)." " .
                "AND user_id = ".$db->quote($GLOBALS['user']->id)." " .
            "ORDER BY score DESC " .
            "LIMIT 1 " .
        "")->fetch(PDO::FETCH_COLUMN, 0);
        if ($my_score) {
            $something_new = $db->query(
                "SELECT COUNT(*) " .
                "FROM st_score " .
                "WHERE seminar_id = ".$db->quote($course_id)." " .
                    "AND mkdate > ".$db->quote($last_visit)." " .
                    "AND user_id != ".$db->quote($GLOBALS['user']->id)." " .
                    "AND score > ".$db->quote($my_score)." " .
            "")->fetch(PDO::FETCH_COLUMN, 0);
        }
        if ($something_new) {
            $navigation->setImage($this->getPluginURL()."/assets/fighter_red.png", array('title' => $this->getDisplayName()));
        } else {
            $navigation->setImage($this->getPluginURL()."/assets/fighter_grey.png", array('title' => $this->getDisplayName()));
        }
        return $navigation;
    }
    
    public function getInfoTemplate($course_id) {
        
    }
    
    protected function getTemplate($template_file_name, $layout = "without_infobox") {
        if ($layout) {
            if (Navigation::hasItem("/course/survivaltraining") && Navigation::getItem("/course/survivaltraining")->isActive()) {
                PageLayout::setTitle($GLOBALS['SessSemName'][0] .": ".$this->getDisplayName());
            } else {
                PageLayout::setTitle($this->getDisplayName());
            }
        }
        if (!$this->template_factory) {
            $this->template_factory = new Flexi_TemplateFactory(dirname(__file__)."/templates");
        }
        $template = $this->template_factory->open($template_file_name);
        if ($layout) {
            $template->set_layout($GLOBALS['template_factory']->open($layout === "without_infobox" ? 'layouts/base_without_infobox' : 'layouts/base'));
        }
        return $template;
    }
    
}